import defaultHandlers from './defaultHandlers';
export declare type Handler<Value> = (prevValue: Value) => (...args: any[]) => Value;
declare type Omit<T, K extends keyof T> = Pick<T, Exclude<keyof T, K>>;
declare type HandlerMethods<Handler extends any> = {
    [key in keyof Handler]: ReturnType<Handler[key]>;
};
declare type ID = number;
interface ContextScope {
    value: any;
    initialized: boolean;
}
declare class Context<Value extends any, CustomHandlers = {
    [key: string]: Handler<Value>;
}> {
    private readonly options;
    private scopes;
    getScope: (id: number) => ContextScope;
    private instance;
    private regex;
    constructor(options?: {
        initialValue?: Value;
        customHandlers?: CustomHandlers;
        strictMode?: boolean;
    });
    /**
     * Creates a new context scope
     * @param callback The callback to be executed inside the scope
     */
    scope<Handlers extends HandlerMethods<CustomHandlers & Omit<typeof defaultHandlers, 'set'>>, CurrentScope extends {
        set: ((newValue: Value | (((prevValue: Value) => Value))) => Value);
        id: ID;
        value: Value;
        destroy: () => void;
    } & Handlers, Callback extends (scope: CurrentScope) => any>(callback: Callback): ReturnType<Callback>;
    /**
     * Gets the value of the current scope from stack
     */
    readonly value: any;
    /**
     * Gets the value of the current scope from stack
     */
    getValue(throwOnNoScope?: boolean): any;
    /**
     * Destroys a scope from memory
     */
    destroy(id: ID): void;
    /**
     * Gets the scopes from the current stack
     */
    getScopes(): number[];
    /**
     * Gets the value for a given stack trace
     */
    private getValueFromTrace;
    /**
     * Gets the scopes from a given stack trace
     */
    private getScopesFromTrace;
    /**
     * Extracts the ID from a trace
     */
    private getIDFromTrace;
    /**
     * Throws an error (or warns when not in strict mode)
     */
    private error;
    cleanup(): void;
}
export default Context;
//# sourceMappingURL=index.d.ts.map