"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var memoizee_1 = __importDefault(require("memoizee"));
var constants_1 = require("./constants");
var defaultHandlers_1 = __importDefault(require("./defaultHandlers"));
var scope_1 = __importDefault(require("./scope"));
var util_1 = require("./util");
var instances = 0;
var Context = /** @class */ (function () {
    function Context(options) {
        if (options === void 0) { options = {}; }
        var _this = this;
        this.options = options;
        this.scopes = [];
        this.getScope = function (id) { return _this.scopes[id]; };
        this.instance = ++instances;
        this.regex = new RegExp(" +at Function\\." + util_1.escapeRegex(constants_1.SCOPE, this.instance) + "([0-8" + util_1.escapeRegex(util_1.SEPARATOR) + "]+)");
        /**
         * Gets the value for a given stack trace
         */
        this.getValueFromTrace = memoizee_1.default(function (trace, throwOnNoScope) {
            var initial = 'initialValue' in _this.options;
            var scopes = _this.getScopesFromTrace(trace);
            var wasUninitialized = false;
            for (var _i = 0, scopes_1 = scopes; _i < scopes_1.length; _i++) {
                var id = scopes_1[_i];
                var scope = _this.getScope(id);
                if (scope) {
                    if (scope.initialized)
                        return scope;
                    wasUninitialized = true;
                    break;
                }
            }
            if (throwOnNoScope &&
                (scopes.length === 0 || (wasUninitialized && !initial)))
                _this.error(scopes.length === 0 ? constants_1.NO_SCOPE : constants_1.UNINITIALIZED_SCOPE);
            return { value: _this.options.initialValue, initialized: false };
        });
        /**
         * Extracts the ID from a trace
         */
        this.getIDFromTrace = memoizee_1.default(function (trace) {
            var match = trace.match(_this.regex);
            if (!match)
                return null;
            var _a = match[1].split(util_1.SEPARATOR), data = _a.slice(1);
            var id = +data[0];
            return id;
        });
    }
    /**
     * Creates a new context scope
     * @param callback The callback to be executed inside the scope
     */
    Context.prototype.scope = function (callback) {
        var _this = this;
        var id = this.scopes.push({ value: undefined, initialized: false }) - 1;
        var controller = new scope_1.default(this.getValue(false));
        controller.onChange = function (value, method) {
            var scope = _this.getScope(id);
            if (!scope)
                return _this.error("" + constants_1.SCOPE_DESTROYED(id) + (method ? constants_1.SCOPE_DESTROYED_METHOD(method) : ''));
            scope.initialized = true;
            scope.value = value;
        };
        var scope = __assign({ id: id,
            get value() {
                return controller.value;
            }, destroy: function () { return _this.destroy(id); } }, controller.handleMethods(__assign({}, this.options.customHandlers, defaultHandlers_1.default)));
        var wrapper = util_1.wrap(constants_1.SCOPE, this.instance, id);
        var result = wrapper.call(callback, scope);
        return result;
    };
    Object.defineProperty(Context.prototype, "value", {
        /**
         * Gets the value of the current scope from stack
         */
        get: function () {
            return this.getValue(true);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Gets the value of the current scope from stack
     */
    Context.prototype.getValue = function (throwOnNoScope) {
        if (throwOnNoScope === void 0) { throwOnNoScope = true; }
        var trace = util_1.getTrace();
        return this.getValueFromTrace(trace, throwOnNoScope).value;
    };
    /**
     * Destroys a scope from memory
     */
    Context.prototype.destroy = function (id) {
        delete this.scopes[id];
    };
    /**
     * Gets the scopes from the current stack
     */
    Context.prototype.getScopes = function () {
        var trace = util_1.getTrace();
        return this.getScopesFromTrace(trace);
    };
    /**
     * Gets the scopes from a given stack trace
     */
    Context.prototype.getScopesFromTrace = function (trace) {
        var _this = this;
        var traces = util_1.parseTrace(trace);
        var scopes = [];
        traces.forEach(function (trace) {
            var id = _this.getIDFromTrace(trace);
            if (id !== null)
                scopes.push(id);
        });
        return scopes;
    };
    /**
     * Throws an error (or warns when not in strict mode)
     */
    Context.prototype.error = function (message) {
        if (this.options.strictMode !== false) {
            throw new Error(message);
        }
        console.warn('[context-scope]', message);
    };
    Context.prototype.cleanup = function () {
        this.getValueFromTrace.clear();
        this.getIDFromTrace.clear();
    };
    return Context;
}());
exports.default = Context;
